<!------------------------------------------------------------------------------
dump.cfm
Used as a replacement for cfclasses/dump.cfm which can not be used in a shared
hosting environment due to its reliance on createobject(JAVA) which is not
available in a secure sandbox.

Copyright (c) 2006-8, Jochem van Dieten
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, 
	  this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, 
	  this list of conditions and the following disclaimer in the documentation 
	  and/or other materials provided with the distribution.
    * Neither the name of the author nor the names of its contributors 
	  may be used to endorse or promote products derived from this software 
	  without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

------------------------------------------------------------------------------->

<cfif StructKeyExists(attributes, "var")>
	<cfoutput>
		<style type="text/css">
		##customDump .struct {
			text-align: left;
			vetical-align: top;
		}
		##customDump table .header {
			background-color: ##CCCCCC;
		}
		##customDump .struct th {
			vetical-align: top;
		}
		##customDump table, tr, th, td {
			border: 1px black solid;
			border-collapse: collapse;
		}
		##customDump td {
			padding: 2px 1em;
		}
		</style>
		<table id="customDump">
			<cfif StructKeyExists(attributes, "label")>
				<tr class="header">
					<th>#attributes.label#</th>
				</tr>
			</cfif>
			<tr>
				<td>#dump(attributes.var)#</td>
			</tr>
		</table>
	</cfoutput>
</cfif>


<cffunction name="dump" output="true" returntype="void">
	<cfargument name="element" type="any" required="true" />

	<cfif IsQuery(arguments.element)>
		<cfset dumpQuery(arguments.element) />
	<cfelseif IsObject(arguments.element)>
		<cfset dumpObject(arguments.element) />
	<cfelseif IsStruct(arguments.element)>
		<cfset dumpStruct(arguments.element) />
	<cfelseif IsArray(arguments.element)>
		<cfset dumpArray(arguments.element) />
	<cfelseif IsSimpleValue(arguments.element)>
		<cfoutput>#arguments.element#</cfoutput>
	<cfelseif IsCustomFunction(arguments.element)>
		<cfset dumpFunction(arguments.element) />
	<cfelse>
		<cfdump var="#arguments.element#" />
	</cfif>

</cffunction>

<cffunction name="dumpQuery" output="true" returntype="void">
	<cfargument name="inQuery" type="query" required="true" />
	
	<cfset var column = "" />
	
	<cfoutput>
		<table class="table">
		<tr class="header">
			<th colspan="#ListLen(arguments.inQuery.columnlist)#">Query</th>
		</tr>
		<tr>
		<cfloop list="#arguments.inQuery.columnlist#" index="column">
			<th>#column#</th>
		</cfloop>
		</tr>
	<cfloop query="inQuery">
		<tr>
		<cfloop list="#inQuery.columnlist#" index="column">
			<td>#dump(arguments.inQuery[column][currentRow])#</td>
		</cfloop>
		</tr>
	</cfloop>
	</table></cfoutput>

</cffunction>

<cffunction name="dumpStruct" output="true" returntype="void">
	<cfargument name="inStruct" type="struct" required="true" />
	
	<cfset var key = "" />
	
	<cfoutput>
		<table class="struct">
		<tr class="header">
			<th colspan="2">Struct</th>
		</tr>
		<cfloop collection="#arguments.instruct#" item="key">
			<tr>
				<th>#key#</th>
				<td>#dump(arguments.inStruct[key])#</td>
			</tr>
		</cfloop>
		</table>
	</cfoutput>
	
</cffunction>

<cffunction name="dumpArray" output="true" returntype="void">
	<cfargument name="inArray" type="array" required="true" />
	
	<cfset var i = "" />
	
	<cfoutput>
		<table class="array">
		<tr class="header">
			<th colspan="2">Array</th>
		</tr>
		<cfloop from="1" to="#arrayLen(arguments.inArray)#" index="i">
			<tr>
				<th>#i#</th>
				<td>#dump(arguments.inArray[i])#</td>
			</tr>
		</cfloop>
		</table>
	</cfoutput>
	
</cffunction>

<cffunction name="dumpObject" output="true" returntype="void">
	<cfargument name="inObject" required="true" />

	<cfset var objectMetaData = GetMetaData(arguments.inObject) />
	<cfoutput>
		<table class="object">
			<tr class="header">
				<th colspan="2">Object #objectMetaData.name#</th>
			</tr>
			<cfif StructKeyExists(objectMetaData, "path")>
			<tr>
				<td>Path</td>
				<td>#objectMetaData.path#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(objectMetaData, "fullname")>
			<tr>
				<td>Fullname</td>
				<td>#objectMetaData.fullname#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(objectMetaData, "implements")>
			<tr>
				<td>Implements</td>
				<td>
					<table>
						<tr>
							<th>Name</th>
							<th>Path</th>
						</tr>
					<cfloop collection="#objectMetaData.implements#" item="interface">
						<tr>
							<td>#interface#</td>
							<td>#objectMetaData.implements[interface]["path"]#</td>
						</tr>
					</cfloop>
					</table>
				</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(objectMetaData, "extends")>
			<tr>
				<td>Extends</td>
				<td>#dump(objectMetaData.fullname)#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(objectMetaData, "functions")>
			<tr>
				<td>Methods</td>
				<td>
					<table>
					<cfloop array="#objectMetaData.functions#" index="function">
						<tr>
							<td>#dumpFunction(inObject[function.name])#</td>
						</tr>
					</cfloop>
					</table>
				</td>
			</tr>
			</cfif>
		</table>
	</cfoutput>
</cffunction>

<cffunction name="dumpFunction" output="true" returntype="void">
	<cfargument name="inFunction" required="true" />

	<cfset var functionmetaData = GetMetaData(arguments.inFunction) />
	<cfoutput>
		<table class="function">
			<tr class="header">
				<th colspan="2">Function #functionmetaData.name#</th>
			</tr>
			<cfif StructKeyExists(functionmetaData, "Access")>
			<tr>
				<td>Access</td>
				<td>#functionmetaData.Access#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(functionmetaData, "displayname")>
			<tr>
				<td>displayname</td>
				<td>#functionmetaData.displayname#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(functionmetaData, "output")>
			<tr>
				<td>output</td>
				<td>#functionmetaData.output#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(functionmetaData, "returntype")>
			<tr>
				<td>returntype</td>
				<td>#functionmetaData.returntype#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(functionmetaData, "roles")>
			<tr>
				<td>roles</td>
				<td>#functionmetaData.roles#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(functionmetaData, "parameters") AND ArrayLen(functionmetaData.parameters)>
			<tr>
				<td>parameters</td>
				<td>#dump(functionmetaData.parameters)#</td>
			</tr>
			</cfif>
			<cfif StructKeyExists(functionmetaData, "userMetadata")>
			<tr>
				<td>userMetadata</td>
				<td>#dump(functionmetaData.userMetadata)#</td>
			</tr>
			</cfif>
		</table>
	</cfoutput>
</cffunction>
